==> tin-1.05.tar.gz: tin-1.05/search.c <==
/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : search.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 26-08-91
 *  Release   : 1.0
 *  Notes     :
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern FILE *note_fp;
extern int cur_groupnum;
extern int first_group_on_screen;
extern int last_group_on_screen;
extern int first_subj_on_screen;
extern int last_subj_on_screen;
extern int index_point;
extern int note_line;
extern int note_page;
extern int note_end;
extern long note_mark[MAX_PAGES];

/*
 * last search patterns
 */

char author_search_string[LEN];
char group_search_string[LEN];
char subject_search_string[LEN];
char art_search_string[LEN];


/*
 *  group.c & page.c
 */
 
int search_author (current_art, forward)
	int current_art;
	int forward;
{
	char buf[LEN];
	char buf2[LEN];
	int i;

	clear_message();

	if (forward) {
		sprintf (buf2, txt_author_search_forwards, author_search_string);
	} else {
		sprintf (buf2, txt_author_search_backwards, author_search_string);
	}

	
	if (! parse_string (buf2, buf)) {
		return -1;
	}
	
	if (strlen (buf)) {
		strcpy (author_search_string, buf);
	} else {
		if (author_search_string[0]) {
			strcpy (buf, author_search_string);
		} else {
			info_message (txt_no_search_string);	
			return -1;
		}
	}

	wait_message (txt_searching);

	make_lower(author_search_string, buf);

	i = current_art;

	do {
		if (forward) {
			i = next_response(i);
			if (i < 0)
				i = 0;
		} else {
			i = prev_response(i);
			if (i < 0)
				i = top - 1;
		}

		make_lower(arts[i].from, buf2);
		if (str_str (buf2, buf) != 0) {
			clear_message ();
			return i;
		}
	} while (i != current_art);

	info_message(txt_no_match);
	return -1;
}

/*
 * select.c
 */
 
void search_group (forward)
	int forward;
{
	char buf[LEN];
	char buf2[LEN];
	int i;

	clear_message();

	if (forward) {
		sprintf (buf2, txt_search_forwards, group_search_string);
	} else {
		sprintf (buf2, txt_search_backwards, group_search_string);
	}

	if (! parse_string (buf2, buf)) {
		return;
	}

	if (strlen (buf)) {
		strcpy (group_search_string, buf);
	} else {
		if (group_search_string[0]) {
			strcpy (buf, group_search_string);
		} else {
			info_message (txt_no_search_string);	
			return;
		}
	}

	wait_message (txt_searching);

	i = cur_groupnum;

	make_lower(group_search_string, buf);

	do {
		if (forward)
			i++;
		else
			i--;

		if (i >= local_top)
			i = 0;
		if (i < 0)
			i = local_top - 1;

		make_lower(active[my_group[i]].name, buf2);
		if (str_str (buf2, buf) != 0) {
			if (i >= first_group_on_screen
			&&  i < last_group_on_screen) {
				clear_message ();
				erase_group_arrow();
				cur_groupnum = i;
				draw_group_arrow();
			} else {
				cur_groupnum = i;
				group_selection_page();
			}
			return;
		}
	} while (i != cur_groupnum);

	info_message(txt_no_match);
}

/*
 * group.c
 */

void search_subject (forward, group)
	int forward;
	char *group;
{
	char buf[LEN];
	char buf2[LEN];
	int i, j;

	if (index_point < 0) {
		info_message (txt_no_arts);
		return;
	}
	
	clear_message();

	if (forward) {
		sprintf (buf2, txt_search_forwards, subject_search_string);
	} else {
		sprintf (buf2, txt_search_backwards, subject_search_string);
	}

	if (! parse_string (buf2, buf)) {
		return;
	}

	if (strlen (buf)) {
		strcpy (subject_search_string, buf);
	} else {
		if (subject_search_string[0]) {
			strcpy (buf, subject_search_string);
		} else {
			info_message (txt_no_search_string);	
			return;
		}
	}

	wait_message (txt_searching);

	i = index_point;

	make_lower(subject_search_string, buf);

	do {
		if (forward)
			i++;
		else
			i--;

		if (i >= top_base)
			i = 0;
		if (i < 0)
			i = top_base - 1;

		j = (int) base[i];
		make_lower(arts[j].subject, buf2);
		if (str_str (buf2, buf) != 0) {
			if (i >= first_subj_on_screen
			    &&  i < last_subj_on_screen) {
				clear_message ();
				erase_subject_arrow();
				index_point = i;
				draw_subject_arrow();
			} else {
				index_point = i;
				show_group_page(group);
			}
			return;
		}
	} while (i != index_point);

	info_message(txt_no_match);
}

/*
 *  page.c (search article body)
 */

int search_article (forward)
	int forward;
{
	char buf[LEN];
	char buf2[LEN];
	char string[LEN];
	char pattern[LEN];
	char *p, *q;
	int ctrl_L;
	int i, j;
	int orig_note_end;
	int orig_note_page;

	clear_message ();

	if (forward) {
		sprintf (buf2, txt_search_forwards, art_search_string);
	} else {
		sprintf (buf2, txt_search_backwards, art_search_string);
	}

	if (! parse_string (buf2, buf)) {
		return FALSE;
	}

	if (strlen (buf)) {
		strcpy (art_search_string, buf);
	} else {
		if (art_search_string[0]) {
			strcpy (buf, art_search_string);
		} else {
			info_message (txt_no_search_string);	
			return FALSE;
		}
	}

	make_lower (art_search_string, pattern);
	/*
	 *  save current position in article
	 */
	orig_note_end = note_end;
	orig_note_page = note_page;

	wait_message (txt_searching);
	
	while (! note_end) {
		note_line = 1;
		ctrl_L = FALSE;

		if (note_page == 0) {
			note_line += 4;
		} else {
			note_line += 2;
		}
		while (note_line < LINES) {
			if (fgets(buf, sizeof buf, note_fp) == NULL) {
				note_end = TRUE;
				break;
			}
			buf[LEN-1] = '\0';
			for (p = buf, q = buf2;	*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == '\f') {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j) {
						*q++ = ' ';
					}
				} else if (((*p) & 0xFF) < ' ') {
					*q++ = '^';
					*q++ = ((*p) & 0xFF) + '@';
				} else {
					*q++ = *p;
				}
			}
			*q = '\0';

			make_lower(buf2, string);

			if (str_str (string, pattern) != 0) {
				fseek (note_fp, note_mark[note_page], 0);
				return TRUE;
			}

			note_line += ((int) strlen(buf2) / COLS) + 1;

			if (ctrl_L) {
				break;
			}
		}
		if (! note_end) {
			note_mark[++note_page] = ftell(note_fp);
		}
	}

	note_end = orig_note_end;
	note_page = orig_note_page;
	fseek (note_fp, note_mark[note_page], 0);
	info_message (txt_no_match);
	return FALSE;
}

/*
 * ANSI C strstr () - Use Boyer-Moore algorithm. Downloaded from net.
 */
 
char *str_str (text, pattern)
	char *text;
	char *pattern;
{
	register unsigned char *p, *t;
	register int i, p1, j, *delta;
	int deltaspace[256];
	int patlen;
	int textlen;

	textlen= strlen (text);
	patlen = strlen (pattern);

	/* algorithm fails if pattern is empty */
	if ((p1 = patlen) == 0)
		return (text);

	/* code below fails (whenever i is unsigned) if pattern too long */
	if (p1 > textlen)
		return (NULL);

	/* set up deltas */
	delta = deltaspace;
	for (i = 0; i <= 255; i++)
		delta[i] = p1;
	for (p = (unsigned char *) pattern, i = p1; --i > 0;)
		delta[*p++] = i;

	/*
	 * From now on, we want patlen - 1.
	 * In the loop below, p points to the end of the pattern,
	 * t points to the end of the text to be tested against the
	 * pattern, and i counts the amount of text remaining, not
	 * including the part to be tested.
	 */
	p1--;
	p = (unsigned char *) pattern + p1;
	t = (unsigned char *) text + p1;
	i = textlen - patlen;
	for (;;) {
		if (*p == *t && memcmp((p - p1), (t - p1), p1) == 0)
			return ((char *)t - p1);
		j = delta[*t];
		if (i < j)
			break;
		i -= j;
		t += j;
	}
	return (NULL);
}


void make_lower (s, t)
	char *s;
	char *t;
{

	while (*s) {
		if (isupper(*s))
			*t = tolower(*s);
		else
			*t = *s;
		s++;
		t++;
	}
	*t = 0;
}

==> tin-1.10.tar.gz: tin-1.1/search.c <==
/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : search.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 31-01-92
 *  Notes     :
 *  Copyright : (c) Copyright 1991-92 by Rich Skrenta & Iain Lea
 *              You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern FILE *note_fp;
extern int cur_groupnum;
extern int first_group_on_screen;
extern int last_group_on_screen;
extern int first_subj_on_screen;
extern int last_subj_on_screen;
extern int index_point;
extern int note_line;
extern int note_page;
extern int note_end;
extern long note_mark[MAX_PAGES];

/*
 * last search patterns
 */

char author_search_string[LEN];
char group_search_string[LEN];
char subject_search_string[LEN];
char art_search_string[LEN];


/*
 *  group.c & page.c
 */
 
int search_author (max_art, current_art, forward)
	int max_art;
	int current_art;
	int forward;
{
	char buf[LEN];
	char buf2[LEN];
	int i, patlen;

	clear_message ();

	if (forward) {
		sprintf (buf2, txt_author_search_forwards, author_search_string);
	} else {
		sprintf (buf2, txt_author_search_backwards, author_search_string);
	}
	
	if (! prompt_string (buf2, buf)) {
		return -1;
	}
	
	if (strlen (buf)) {
		strcpy (author_search_string, buf);
	} else {
		if (author_search_string[0]) {
			strcpy (buf, author_search_string);
		} else {
			info_message (txt_no_search_string);	
			return -1;
		}
	}

	wait_message (txt_searching);

	make_lower (author_search_string, buf);

	patlen = strlen (author_search_string);

	i = current_art;

	do {
		if (forward) {
			i = next_response (i);
			if (i < 0)
				i = 0;
		} else {
			i = prev_response (i);
			if (i < 0)
				i = max_art - 1;
		}

		if (arts[i].name == (char *) 0) {
			make_lower (arts[i].from, buf2);
		} else {
			sprintf (msg, "%s (%s)", arts[i].from, arts[i].name);
			make_lower (msg, buf2);
		}

		if (str_str (buf2, buf, patlen) != 0) {
			clear_message ();
			return i;
		}
	} while (i != current_art);

	info_message (txt_no_match);
	return -1;
}

/*
 * select.c
 */
 
void search_group (forward)
	int forward;
{
	char buf[LEN];
	char buf2[LEN];
	int i, patlen;

	clear_message ();

	if (forward) {
		sprintf (buf2, txt_search_forwards, group_search_string);
	} else {
		sprintf (buf2, txt_search_backwards, group_search_string);
	}

	if (! prompt_string (buf2, buf)) {
		return;
	}

	if (strlen (buf)) {
		strcpy (group_search_string, buf);
	} else {
		if (group_search_string[0]) {
			strcpy (buf, group_search_string);
		} else {
			info_message (txt_no_search_string);	
			return;
		}
	}

	wait_message (txt_searching);

	make_lower (group_search_string, buf);

	patlen = strlen (group_search_string);

	i = cur_groupnum;

	do {
		if (forward)
			i++;
		else
			i--;

		if (i >= group_top)
			i = 0;
		if (i < 0)
			i = group_top - 1;

		make_lower (active[my_group[i]].name, buf2);

		if (str_str (buf2, buf, patlen) != 0) {
			if (i >= first_group_on_screen
			&&  i < last_group_on_screen) {
				clear_message ();
				erase_group_arrow ();
				cur_groupnum = i;
				draw_group_arrow ();
			} else {
				cur_groupnum = i;
				group_selection_page ();
			}
			return;
		}
	} while (i != cur_groupnum);

	info_message (txt_no_match);
}

/*
 * group.c
 */

void search_subject (forward, group)
	int forward;
	char *group;
{
	char buf[LEN];
	char buf2[LEN];
	int i, j, patlen;

	if (index_point < 0) {
		info_message (txt_no_arts);
		return;
	}
	
	clear_message ();

	if (forward) {
		sprintf (buf2, txt_search_forwards, subject_search_string);
	} else {
		sprintf (buf2, txt_search_backwards, subject_search_string);
	}

	if (! prompt_string (buf2, buf)) {
		return;
	}

	if (strlen (buf)) {
		strcpy (subject_search_string, buf);
	} else {
		if (subject_search_string[0]) {
			strcpy (buf, subject_search_string);
		} else {
			info_message (txt_no_search_string);	
			return;
		}
	}

	wait_message (txt_searching);

	make_lower (subject_search_string, buf);

	patlen = strlen (subject_search_string);

	i = index_point;

	do {
		if (forward)
			i++;
		else
			i--;

		if (i >= top_base)
			i = 0;
		if (i < 0)
			i = top_base - 1;

		j = (int) base[i];

		make_lower (arts[j].subject, buf2);

		if (str_str (buf2, buf, patlen) != 0) {
			if (i >= first_subj_on_screen
			    &&  i < last_subj_on_screen) {
				clear_message ();
				erase_subject_arrow ();
				index_point = i;
				draw_subject_arrow ();
			} else {
				index_point = i;
				show_group_page (group);
			}
			return;
		}
	} while (i != index_point);

	info_message (txt_no_match);
}

/*
 *  page.c (search article body)
 */

int search_article (forward)
	int forward;
{
	char buf[LEN];
	char buf2[LEN];
	char string[LEN];
	char pattern[LEN];
	char *p, *q;
	int ctrl_L;
	int i, j, patlen;
	int orig_note_end;
	int orig_note_page;

	clear_message ();

	if (forward) {
		sprintf (buf2, txt_search_forwards, art_search_string);
	} else {
		sprintf (buf2, txt_search_backwards, art_search_string);
	}

	if (! prompt_string (buf2, buf)) {
		return FALSE;
	}

	if (strlen (buf)) {
		strcpy (art_search_string, buf);
	} else {
		if (art_search_string[0]) {
			strcpy (buf, art_search_string);
		} else {
			info_message (txt_no_search_string);	
			return FALSE;
		}
	}

	wait_message (txt_searching);
	
	make_lower (art_search_string, pattern);

	patlen = strlen (art_search_string);

	/*
	 *  save current position in article
	 */
	orig_note_end = note_end;
	orig_note_page = note_page;
	
	while (! note_end) {
		note_line = 1;
		ctrl_L = FALSE;

		if (note_page == 0) {
			note_line += 4;
		} else {
			note_line += 2;
		}
		while (note_line < LINES) {
			if (fgets (buf, sizeof buf, note_fp) == NULL) {
				note_end = TRUE;
				break;
			}
			buf[LEN-1] = '\0';
			for (p = buf, q = buf2;	*p && *p != '\n' && q<&buf2[LEN]; p++) {
				if (*p == '\b' && q > buf2) {
					q--;
				} else if (*p == '\f') {		/* ^L */
					*q++ = '^';
					*q++ = 'L';
					ctrl_L = TRUE;
				} else if (*p == '\t') {
					i = q - buf2;
					j = (i|7) + 1;

					while (i++ < j) {
						*q++ = ' ';
					}
				} else if (((*p) & 0xFF) < ' ') {
					*q++ = '^';
					*q++ = ((*p) & 0xFF) + '@';
				} else {
					*q++ = *p;
				}
			}
			*q = '\0';

			make_lower (buf2, string);

			if (str_str (string, pattern, patlen) != 0) {
				fseek (note_fp, note_mark[note_page], 0);
				return TRUE;
			}

			note_line += ((int) strlen(buf2) / COLS) + 1;

			if (ctrl_L) {
				break;
			}
		}
		if (! note_end) {
			note_mark[++note_page] = ftell (note_fp);
		}
	}

	note_end = orig_note_end;
	note_page = orig_note_page;
	fseek (note_fp, note_mark[note_page], 0);
	info_message (txt_no_match);
	return FALSE;
}


void make_lower (s, t)
	char *s;
	char *t;
{

	while (*s) {
		if (isupper(*s))
			*t = tolower(*s);
		else
			*t = *s;
		s++;
		t++;
	}
	*t = 0;
}
